class LearningObject::AttachmentBuilder < ::Builder

  ##disable method
  # build attachments with id's is out of scope.
  def self.build(objects = [])
    raise NotImplementedError, 'LearningObject::AttachmentBuilder#build is out of scope.'
  end

  def self.build_from_orientdb(args = {})
    args['retrieve_link'] = args.delete('retrieveLink')
    args['mime_type'] = args.delete('mimeType')
    build_from_cache args do
      ::LearningObject::Attachment.new(args)
    end
  end

  private

  def self.build_from_cache(args={})
    object = nil

    unless args.nil? || !block_given?
      # cache object when build
      object = Rails.cache.fetch(cache_key(args['id'], args['link']), expires_in: 12.hours) { yield }
    end

    object
  end

  def self.permitted_args(args={})
    permitted = [:id, :name, :link, :retrieve_link, :description, :format, :mime_type, :size]
    args.delete_if do |key, value|
      !permitted.include? key.to_sym
    end
  end

end