class OrientDb::Base
  include OrientDb::Methods::FinderMethods
  include OrientDb::Methods::SanitizableMethods

  def initialize(orientdb_connection)
    @connection = orientdb_connection
  end

  def connection
    @connection
  end

  def find(id)
    result = get_by_rid(CGI::unescape(id))
    build_object result
  end

  def all
    objects_hash = connection.query "SELECT FROM #{odb_class}", :limit => -1
    objects = build_objects(objects_hash) || []
  end

  def all_from_offset_to_limit(offset, limit)
    objects_hash = connection.query "SELECT FROM #{odb_class} LIMIT #{limit} SKIP #{offset}", :limit => limit
    objects = build_objects(objects_hash) || []
  end

  def destroy(object)
    connection.command sprintf("DELETE VERTEX %s", object.id)
  end

  def create(object)
    if object.valid?
      hash = object.to_orientdb_hash
      result = connection.create_document(hash)
      object.id = result["@rid"]
    end
    object
  end

  def update_property(object, property, value)
    if accepted_properties.include? property
      connection.command "UPDATE #{odb_class} SET #{property}='#{value}', last_modified='#{Time.now()}' WHERE @rid = #{object.id}"
    end
  end

  protected

  def build_objects(hash=[])
    objects = []
    hash.each do |h|
      objects << build_object(h)
    end
    objects
  end

  def build_object(args={})
    builder = "#{args['@class']}Builder".constantize
    builder.build_from_orientdb args unless args['@class'].blank? || builder.nil?
  end

  def accepted_properties
    raise NoMethodError, "You must implement this method"
  end

  def odb_class
    fullname = self.class.to_s.split('Repository').first
    fullname.split('::').last.to_s
  end

end
