module OrientDb
  class LearningObjectRepository < Base
    include OrientDb::Methods::SocialMethods
    include OrientDb::Methods::EdgeMethods
    include OrientDb::Methods::CountableMethods
    include RepositoriesProxy

    #get the ten most visualised. It's probably making too many useless accesses to orientDB...
    def get_most_visualised
      connection.query("SELECT FROM (SELECT name, @rid,in('Views').size() AS views FROM LearningObject) ORDER BY views DESC LIMIT 10")
    end

    def get_publisher(learning_object)
      result = get_edges_end("PublishedBy", "out", learning_object.id)
      result = result.first

      unless result.blank?
        if result["@class"] == "Institution"
          institution_repository.build_object(result)
        else
          user_repository.find(result['p_id'])
        end
      end
    end

    # Usage:
    #   learning_object = repository.for(:learning_objects).get_by_dspace_id 123
    #
    def get_by_dspace_id(id_dspace)
      result = select_by_property(odb_class, "id_dspace", id_dspace)
      build_object result.first
    end

    #all method returns all learning objects of an user
    def all(user)
      build_objects(get_edges_end('PublishedBy', 'in', user.rid) || [])
    end

    def all_by_score_from_offset_to_limit(offset, limit)
      return [] if offset.blank? || limit.blank?

      # expensive query
      key = 'learning_objects/all/score/' + offset.to_s + '/' + limit.to_s
      objects_hash = Rails.cache.fetch(key, expires_in: 6.hours) do
        objects = connection.query "SELECT FROM LearningObject ORDER BY score DESC LIMIT #{limit} SKIP #{offset}", limit: limit
        objects.map { |o| {'rid' => o['@rid']} }
      end

      LearningObjectBuilder.build(objects_hash)
    end

    def get_subjects(learning_object)
      result = get_edges_end("IsAbout", "out", learning_object.id)
      subject_repository.build_objects(result)
    end

    def get_attributes(learning_object)
      attribute_repository.find_all_by_learning_object learning_object
    end

    def create_relations(learning_object)
      relations = OrientDb::LearningObject::RelationsChain.new(learning_object, connection)
      relations.create
    end

    def save(learning_object)
      if learning_object.valid?
        connection.command "INSERT INTO LearningObject CONTENT #{learning_object.to_json}"
      end
    end

    def types
      Rails.cache.fetch("learning_object/types", expires_in: 1.days) do
        query = "SELECT DISTINCT(type) FROM LearningObject GROUP BY type"
        types_hash = connection.query query, limit: 100
        # return array with types
        types_hash.map do |e|
          e['type']
        end
      end
    end

  end
end
