# This proxy will keep postgres and orientdb syncronized
#   When some client call UserRepositoryProxy, it will delegate the missing methods for ActiveRecord User model
module RepositoriesProxy

  def method_missing(method_name, *arguments, &block)
    if is_repository? method_name
      application_repository[repository_name(method_name)]
    else
      super
    end
  end

  def respond_to?(method_name, include_private = false)
    is_repository?(method_name) || super
  end

  private

  def is_repository?(method_name)
    (method_name.to_s =~ /_repository/) && application_repository.include?(repository_name(method_name))
  end

  def repository_name(method_name)
    method_name.to_s.split('_repository').first.to_sym
  end

  def application_repository
    Portalmec::Application.repository
  end

end
